@@ -1,7 +1,7 @@
 package IPC::System::Options;
 
-our $DATE = '2016-06-08'; # DATE
-our $VERSION = '0.27'; # VERSION
+our $DATE = '2016-10-06'; # DATE
+our $VERSION = '0.28'; # VERSION
 
 use strict;
 use warnings;
@@ -42,7 +42,7 @@ sub _quote {
         require Win32::ShellQuote;
         return Win32::ShellQuote::quote_system_string(@_);
     } else {
-        return join(" ", map { shell_quote($_) } @_);
+        return shell_quote(@_);
     }
 }
 
@@ -54,8 +54,44 @@ sub _system_or_readpipe_or_run {
     }
     my @args = @_;
 
+    # check known options
+    for (keys %$opts) {
+        die "Unknown option '$_'"
+            unless /\A(
+                        capture_stdout|capture_stderr|
+                        chdir|dies?|env|lang|log||max_log_output|shell|
+                        stdin # XXX: only for run()
+                    )\z/x;
+    }
+
     my $opt_die = $opts->{die} || $opts->{dies};
 
+    my $exit_code;
+    my $os_error = "";
+    my $extra_error;
+
+    $log ||= do { require Log::Any::IfLOG; Log::Any::IfLOG->get_logger } if $opts->{log};
+
+    my $cwd;
+    if ($opts->{chdir}) {
+        require Cwd;
+        $cwd = Cwd::getcwd();
+        if (!defined $cwd) { # checking $! is always true here, why?
+            $log->error("Can't getcwd: $!") if $log;
+            $exit_code = -1;
+            $os_error = $!;
+            $extra_error = "Can't getcwd";
+            goto CHECK_RESULT;
+        }
+        unless (chdir $opts->{chdir}) {
+            $log->error("Can't chdir to '$opts->{chdir}': $!") if $log;
+            $exit_code = -1;
+            $os_error = $!;
+            $extra_error = "Can't chdir";
+            goto CHECK_RESULT;
+        }
+    }
+
     # set ENV
     my %save_env;
     my %set_env;
@@ -74,20 +110,12 @@ sub _system_or_readpipe_or_run {
         }
     }
 
-    $log ||= do { require Log::Any::IfLOG; Log::Any::IfLOG->get_logger } if $opts->{log};
-
     my $wa;
     my $res;
-    my $exit_code;
-    my $os_error;
 
     my $code_capture = sub {
         my $doit = shift;
 
-        if ($opts->{capture}) {
-            die "The 'capture' option has been replaced by 'capture_stdout' & 'capture_stderr', please adjust your code first";
-        }
-
         if ($opts->{capture_stdout} && $opts->{capture_stderr}) {
             require Capture::Tiny;
             (${ $opts->{capture_stdout} }, ${ $opts->{capture_stderr} }) =
@@ -207,31 +235,39 @@ sub _system_or_readpipe_or_run {
         }
     }
 
+    # restore previous working directory
+    if ($cwd) {
+        unless (chdir $cwd) {
+            $log->error("Can't chdir back to '$cwd': $!") if $log;
+            $exit_code ||= -1;
+            $os_error = $!;
+            $extra_error = "Can't chdir back";
+            goto CHECK_RESULT;
+        }
+    }
+
+  CHECK_RESULT:
     if ($exit_code) {
         if ($opts->{log} || $opt_die) {
             my $msg = sprintf(
-                "%s(%s) failed: %d (%s)%s%s",
+                "%s(%s) failed: %s (%s)%s%s",
                 $which,
                 join(" ", @args),
-                $exit_code,
-                explain_child_error($exit_code, $os_error),
+                defined $extra_error ? "" : $exit_code,
+                defined $extra_error ? "$extra_error: $os_error" : explain_child_error($exit_code, $os_error),
                 (ref($opts->{capture_stdout}) ?
                      ", captured stdout: <<" .
-                     (${$opts->{capture_stdout}} // ''). ">>" : ""),
+                     (defined ${$opts->{capture_stdout}} ? ${$opts->{capture_stdout}} : ''). ">>" : ""),
                 (ref($opts->{capture_stderr}) ?
                      ", captured stderr: <<" .
-                     (${$opts->{capture_stderr}} // ''). ">>" : ""),
+                     (defined ${$opts->{capture_stderr}} ? ${$opts->{capture_stderr}} : ''). ">>" : ""),
             );
             $log->error($msg) if $opts->{log};
             die $msg if $opt_die;
         }
     }
 
-    # make sure our user see the correct exit code & os error. this is not
-    # currently needed as nothing yet clobbers $? & $!. but they might be needed
-    # later (if we do more stuff which sets these variables) so i put it in.
     $? = $exit_code;
-    $! = $os_error;
 
     return $wa && $which ne 'run' ? @$res : $res;
 }
@@ -268,7 +304,7 @@ IPC::System::Options - Perl's system() and readpipe/qx replacement, with options
 
 =head1 VERSION
 
-This document describes version 0.27 of IPC::System::Options (from Perl distribution IPC-System-Options), released on 2016-06-08.
+This document describes version 0.28 of IPC::System::Options (from Perl distribution IPC-System-Options), released on 2016-10-06.
 
 =head1 SYNOPSIS
 
@@ -364,6 +400,21 @@ Capture stdout using L<Capture::Tiny>.
 
 Capture stderr using L<Capture::Tiny>.
 
+=item * chdir => str
+
+Attempt to change to specified directory first and change back to the original
+directory after the command has been run.
+
+If the attempt to chdir before command execution fails, will die if C<die>
+option is set to true. Otherwise, C<$!> (OS error) will be set to the C<chdir()>
+error and to minimize surprise C<$?> (child exit code) will also be set to
+non-zero value (-1) even though at this point no child process has been run.
+
+If the attempt to chdir back (after command execution) fails, will die if C<die>
+option is set to true. Otherwise, C<$!> will be set to the C<chdir()> error and
+C<$?> will be set to -1 only if C<$?> is zero. So if the command fails, C<$?>
+will contain the exit code of the command.
+
 =back
 
 =head2 readpipe([ \%opts ], @args)
@@ -406,6 +457,10 @@ See option documentation in C<system()>.
 If set, will limit result length being logged. It's a good idea to set this
 (e.g. to 1024) if you expect some command to return large output.
 
+=item * chdir => str
+
+See option documentation in C<system()>.
+
 =back
 
 =head2 run([ \%opts ], @args)
@@ -442,6 +497,10 @@ See option documentation in C<system()>.
 
 Supply standard input.
 
+=item * chdir => str
+
+See option documentation in C<system()>.
+
 =back
 
 =head1 HOMEPAGE
